// I2C_Sample.cpp : Defines the entry point for the console application.
//

//#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include <winioctl.h>
#include "i2c.h"

//Thermometer address
#define SLAVE_ADDRESS_EXTENDER	0x22	//0b0100010

// Temprature register address
#define REGISTER_CONFIG0	0x6
// Tlow register address
#define REGISTER_OUTPORT	0x2

/**********************************************************************************************************
 About I2C_GpioExtender Example.
***********************************************************************************************************
 This example shows how to transact with the CAT9555 Gpio extender. The extender communicates thrgough the
 I2C bus.
 All register are set to 0xFF after reset. Another feature we use in this device is the pairing of registers,
 meaning that if you address say Configuration register number 1 and write 2 bytes, then the 1st byte is written
 to configuration register 1 and the 2nd byte is written to configuration register 0.
 In this example we do the following:
 -Read Configuration registers (to see value before setting it)
 -Writing to Configuration registers
 -Read Configuration registers (to confirm that the write was successful - just for this example)
 -Read the Output Port registers (to see value before setting it)
 -Write the Output Port registers
 -Read Output Port registers (to confirm that the write was successful - just for this example)

 When done you will see that all of the pins, except for pin 1, have a logical value 1 (the default value)
 and pin #1 outputs a logical value of 0 (you can use any measuring device you like)
************************************************************************************************************/

int _tmain(int argc, _TCHAR* argv[])
{
	UCHAR ucSlaveAddr;
	DWORD dwData=0;
	I2CTRANS trans;
	BOOL bRet;

	//Open a handle to I2C
	HANDLE hI2C = I2CUserOpen(I2C_BUS_NAME);
	if(hI2C == INVALID_HANDLE_VALUE)
	{
		printf("Error opening I2C handle, errorCode is %d\n", GetLastError());
		return -1;
	}

	//Choose slave address
	//The PXA-270 only supports 7 bit slave address so make sure 
	//the address you have chosen isn't greater than 127...
	ucSlaveAddr = SLAVE_ADDRESS_EXTENDER;

	//Read the configuration register to see that after we will write it
	//It will change
	memset(&trans,0,sizeof(I2CTRANS));
	//We first set the address of the register we want to read from
	//by writing it's address
	trans.mTransactions = 2;
	trans.mDeviceAddr = ucSlaveAddr;
	//Using RegulareMode clock just for this transaction
	trans.mClkSpeed = I2C_CLOCK_REGULAR_MODE;
	//First transaction is choose register
	trans.mOpCode[0] = I2C_OPCODE_WRITE;
	trans.mTransLen[0] = 1;
	trans.mBuffer[0] = REGISTER_CONFIG0;	
	trans.mBufferOffset[0] = 0;
	//2nd transaction is choose register
	trans.mOpCode[1] = I2C_OPCODE_READ;
	trans.mTransLen[1] = 2;
	trans.mBuffer[1] = REGISTER_CONFIG0;	
	trans.mBufferOffset[1] = 1;
	bRet = I2CUserTransact(hI2C, &trans);
	if (!bRet)
	{
		printf("I2C Transaction failed for some reason.");
		return -1;
	}
	printf("Read Configuration before writing to it: High= 0x%02x, Low= 0x%02x\r\n", trans.mBuffer[1],trans.mBuffer[2]);


	//Configure GPIO 0.0 to be Output
	memset(&trans,0,sizeof(I2CTRANS));
	//Writing to the register is done in one transaction
	trans.mTransactions = 1;
	trans.mDeviceAddr = ucSlaveAddr;
	trans.mClkSpeed = I2C_CLOCK_FAST_MODE;
	//First transaction is choose register
	trans.mOpCode[0] = I2C_OPCODE_WRITE;
	trans.mTransLen[0] = 3;
	trans.mBuffer[0] = REGISTER_CONFIG0;	
	trans.mBufferOffset[0] = 0;
	trans.mBuffer[1] = 0xFE;//Setting pin 0.0 as output and all the rest as inputs
	trans.mBuffer[2] = 0xFF;
	bRet = I2CUserTransact(hI2C, &trans);
	if (!bRet)
	{
		printf("I2C Transaction failed for some reason.");
		return -1;
	}

	//Read the configuration register to see that after we will write it
	//It will change
	memset(&trans,0,sizeof(I2CTRANS));
	//We first set the address of the register we want to read from
	//by writing it's address
	trans.mTransactions = 2;
	trans.mDeviceAddr = ucSlaveAddr;
	trans.mClkSpeed = I2C_CLOCK_FAST_MODE;
	//First transaction is choose register
	trans.mOpCode[0] = I2C_OPCODE_WRITE;
	trans.mTransLen[0] = 1;
	trans.mBuffer[0] = REGISTER_CONFIG0;	
	trans.mBufferOffset[0] = 0;
	//2nd transaction is choose register
	trans.mOpCode[1] = I2C_OPCODE_READ;
	trans.mTransLen[1] = 2;
	//trans.mBuffer[1] = REGISTER_CONFIG0;	
	trans.mBufferOffset[1] = 1;
	bRet = I2CUserTransact(hI2C, &trans);
	if (!bRet)
	{
		printf("I2C Transaction failed for some reason.");
		return -1;
	}
	printf("Read Configuration After writing to it: High= 0x%02x, Low= 0x%02x\r\n",trans.mBuffer[1],trans.mBuffer[2]);


	//Read the OUTPUTPORT register to see that after we will write it
	//It will change
	memset(&trans,0,sizeof(I2CTRANS));
	//We first set the address of the register we want to read from
	//by writing it's address
	trans.mTransactions = 2;
	trans.mDeviceAddr = ucSlaveAddr;
	trans.mClkSpeed = I2C_CLOCK_FAST_MODE;
	//First transaction is choose register
	trans.mOpCode[0] = I2C_OPCODE_WRITE;
	trans.mTransLen[0] = 1;
	trans.mBuffer[0] = REGISTER_OUTPORT;	
	trans.mBufferOffset[0] = 0;
	//2nd transaction is choose register
	trans.mOpCode[1] = I2C_OPCODE_READ;
	trans.mTransLen[1] = 2;
	//trans.mBuffer[1] = REGISTER_CONFIG0;	
	trans.mBufferOffset[1] = 1;
	bRet = I2CUserTransact(hI2C, &trans);
	if (!bRet)
	{
		printf("I2C Transaction failed for some reason.");
		return -1;
	}
	printf("Read output port Before writing to it: High= 0x%02x, Low= 0x%02x\r\n",trans.mBuffer[1],trans.mBuffer[2]);

	memset(&trans,0,sizeof(I2CTRANS));
	//Again writing is done in 1 transaction
	trans.mTransactions = 1;
	trans.mDeviceAddr = ucSlaveAddr;
	trans.mClkSpeed = I2C_CLOCK_FAST_MODE;
	//First transaction is choose register
	trans.mOpCode[0] = I2C_OPCODE_WRITE;
	trans.mTransLen[0] = 3;
	trans.mBuffer[0] = REGISTER_OUTPORT;	
	trans.mBufferOffset[0] = 0;
	trans.mBuffer[1] = 0x00;//Pin 0.0 is output, setting to High and all the rest dont matter
	trans.mBuffer[2] = 0x00;
	bRet = I2CUserTransact(hI2C, &trans);
	if (!bRet)
	{
		printf("I2C Transaction failed for some reason.");
		return -1;
	}

	//Let's read the Output port register to see that writing to it changed things
	//(It defaults to 0xFF)
	memset(&trans,0,sizeof(I2CTRANS));
	//We first set the address of the register we want to read from
	//by writing it's address
	trans.mTransactions = 1;
	trans.mDeviceAddr = ucSlaveAddr;
	trans.mClkSpeed = I2C_CLOCK_FAST_MODE;
	//First transaction is choose register
	trans.mOpCode[0] = I2C_OPCODE_READ;
	trans.mTransLen[0] = 2;
	trans.mBufferOffset[0] = 0;
	bRet = I2CUserTransact(hI2C, &trans);
	if (!bRet)
	{
		printf("I2C Transaction failed for some reason.");
		return -1;
	}
	printf("Read Output Port rgisters: OutPort0: 0x%02x, OutPort1: 0x%02x\r\n",trans.mBuffer[0], trans.mBuffer[1]);


	printf("You can now check to see that the pin is high\r\n");


	//Closes the handles
	I2CUserClose(hI2C);
	return 0;
}

