#include <windows.h>
#include <commctrl.h>
#include <winioctl.h>
#include "SPI.h"
#include "gpio.h"


#define SSP1_FREQ				4	//Specified in KHz, accroding to spec needs to be between DC and 5MHz
#define	SSP1_DATA_LEN_EX		16	//Each transaction is 16 bits (writing 16 and reading 16 minimum)
#define	SSP1_DATA_LEN			8	//Each transaction is 8 bits (writing 8 and reading 8 minimum)
#define GPIO_SSP_MANUAL_SFRM	108	//pin number 6 located in P2 on the keypad extender
#define BUFFER_SIZE				5

#define GPIO_GND_22			22	//pin number 10 located in P2 on the keypad extender
#define GPIO_VDD_36			36	//pin number 9 located in P2 on the keypad extender
#define GPIO_VDD_96			96	//pin number 8 located in P2 on the keypad extender
#define GPIO_VDD_91			91	//pin number 7 located in P2 on the keypad extender

#define SPI_PORT_NAME		TEXT("SPI1:")

//All handling of toggling the CLK is done by the MicroController

void ThermometerTransaction16BitsVersion(HANDLE hSPI);
void ThermometerTransaction8BitsVersion(HANDLE hSPI);

int _tmain(int argc, _TCHAR* argv[])
{
	//Set gpio environment
	HANDLE hGpio = GPIOOpen();
	if (hGpio == NULL)
		return FALSE;

	GPIOSetAsOutput(hGpio, GPIO_VDD_36, GPIO_LOW);
	GPIOSetAsOutput(hGpio, GPIO_VDD_96, GPIO_LOW);
	GPIOSetAsOutput(hGpio, GPIO_VDD_91, GPIO_LOW);

	Sleep(30);
	GPIOSetAsOutput(hGpio, GPIO_VDD_36, GPIO_HIGH);
	GPIOSetAsOutput(hGpio, GPIO_VDD_96, GPIO_HIGH);
	GPIOSetAsOutput(hGpio, GPIO_VDD_91, GPIO_HIGH);
	GPIOSetAsOutput(hGpio, GPIO_GND_22, GPIO_LOW);
	GPIOSetAsOutput(hGpio, GPIO_SSP_MANUAL_SFRM, GPIO_LOW);

	Sleep(2000);
	//Start doing the SPI stuff

	//Open a handle to SPI
	//Spi1 is the driver configured to work with SPI1
	//Which is the one port available on EM-270
	HANDLE hSPI= SPIUserOpen(SPI_PORT_NAME);
	if(hSPI == INVALID_HANDLE_VALUE)
	{
		printf("Error opening SPI handle, errorCode is %d\n", GetLastError());
		return -1;
	}
	ThermometerTransaction16BitsVersion(hSPI);
	ThermometerTransaction8BitsVersion(hSPI);
	

	SPIUserClose(hSPI);

	return 0;
}

void ThermometerTransaction16BitsVersion(HANDLE hSPI)
{
	SPI_SLAVE_CONFIG	Config;
	SPITRANS	trans;

	//Initial configuration of the ssp port according to the DS1722+ (MAXIM's) spec
	Config.bCSExternal = TRUE; // Will the SSP port drive the Chip Select or is it an external GPIO we use - Here we use an external GPIO
	Config.bTxTristate = TRUE; // Will TX line be tristated when CS is deasserted
	Config.ChipSelectPol = LOW_INACTIVE; // Polarity of CS
	Config.ClkPhaseHalfCycle = TRUE; 
	Config.ClockPol = LOW_INACTIVE;
	Config.dwClockRate = SSP1_FREQ;
	Config.dwDataSize = SSP1_DATA_LEN_EX;
	Config.dwGpioNum = GPIO_SSP_MANUAL_SFRM;
	SPIUserSetConfiguration(hSPI, &Config); // Let the driver save our configuration
	//Ok now the port is configured and we can start transmitting
	trans.mReadBuffer = new DWORD [BUFFER_SIZE];
	trans.mWriteBuffer = new DWORD [BUFFER_SIZE];
	if (!trans.mReadBuffer || !trans.mWriteBuffer) 
	{
		printf("Failed to allocate memory\r\n");
		goto CleanUp;
	}
	//In one transaction we send information at the first half, and expect data at last 8 bits of the transaction
	trans.mTransactions = 1;
	//Send the address 0x02 to read Temprature MSB
	trans.mWriteBuffer[0] = 0x200;//shifted the info 8 bits because we send MSB first and we want the higher 8 bits to be 0x02
	//Send the transaction
	if (!SPIUserTransact(hSPI, &trans))
	{
		printf("Failed\r\n");
		return ;
	}
	trans.mReadBuffer[0]=0;
	trans.mTransactions = 1;
	//Send the address 0x01 to read Temprature LSB
	trans.mWriteBuffer[0] = 0x100;
	//Send the transaction
	SPIUserTransact(hSPI, &trans);
	//Only 16 bits are non zero at most because data frame size was configured to be 16 bits to read
	//And the hardware padds zeros for the higher bits.

	printf("The temprature is: 0x%08x\r\n", trans.mReadBuffer[0]);

	//Now we will write to the configuration register
	//Using the same configuration
	trans.mReadBuffer[0]=0;
	trans.mWriteBuffer[0] = 0x80F0; //The address is the upper 8 bits and the data is the lower 8 bits

	SPIUserTransact(hSPI, &trans);
	//Let's see if the configuration was written correctly
	trans.mWriteBuffer[0] = 0x000; //the address of the config register when we want to read it
	SPIUserTransact(hSPI, &trans);
	printf("The config reg now is 0x%02x, default after reset is 0xE3",
		trans.mReadBuffer[0]);
	
CleanUp:
	if (trans.mReadBuffer)
		delete [] trans.mReadBuffer;
	if (trans.mWriteBuffer)
		delete [] trans.mWriteBuffer;
}

void ThermometerTransaction8BitsVersion(HANDLE hSPI)
{
	SPI_SLAVE_CONFIG	Config;
	SPITRANS			trans;
	DWORD				dwTemprature;

	//Initial configuration of the ssp port according to the DS1722+ (MAXIM's) spec
	Config.bCSExternal = TRUE; // Will the SSP port drive the Chip Select or is it an external GPIO we use - Here we use an external GPIO
	Config.bTxTristate = TRUE; // Will TX line be tristated when CS is deasserted
	Config.ChipSelectPol = LOW_INACTIVE; // Polarity of CS
	Config.ClkPhaseHalfCycle = TRUE; 
	Config.ClockPol = LOW_INACTIVE;
	Config.dwClockRate = SSP1_FREQ;
	Config.dwDataSize = SSP1_DATA_LEN;
	Config.dwGpioNum = GPIO_SSP_MANUAL_SFRM;
	SPIUserSetConfiguration(hSPI, &Config); // Let the driver save our configuration
	//Ok now the port is configured and we can start transmitting
	trans.mReadBuffer = new DWORD [BUFFER_SIZE];
	trans.mWriteBuffer = new DWORD [BUFFER_SIZE];
	if (!trans.mReadBuffer || !trans.mWriteBuffer) 
	{
		printf("Failed to allocate memory\r\n");
		goto CleanUp;
	}
	trans.mTransactions = 2;
	//Send the address 0x02 to read Temprature MSB
	trans.mWriteBuffer[0] = 0x2;
	//We still send something while we read data - but the device ignores this data
	trans.mWriteBuffer[1] = 0x0;
	//Send the transaction
	SPIUserTransact(hSPI, &trans);
	//The answer is in index 1 because we sent data in cycle 0 and read the data the next cycle
	dwTemprature = trans.mReadBuffer[1]; 

	trans.mReadBuffer[0]=0;trans.mReadBuffer[1]=0;
	trans.mTransactions = 2;
	//Send the address 0x01 to read Temprature LSB
	trans.mWriteBuffer[0] = 0x1;
	//Send the transaction
	SPIUserTransact(hSPI, &trans);
	//NOTICE:Only 8 bits are non zero at most because data frame size was configured to be 8 bits to read
	//And the hardware padds zeros for the higher bits.

	printf("The temprature is: MSB=0x%08x    LSB=0x%08x\r\n", dwTemprature,trans.mReadBuffer[1]);

	//Now we will write to the configuration register
	//Using the same configuration
	trans.mReadBuffer[0]=trans.mReadBuffer[1]=0;
	trans.mWriteBuffer[0] = 0x80; //The address 
	trans.mWriteBuffer[1] = 0xF0; //The data to be written

	SPIUserTransact(hSPI, &trans);
	//Let's see if the configuration was written correctly
	trans.mWriteBuffer[0] = 0x0; //the address of the config register when we want to read it
	trans.mWriteBuffer[1] = 0x0; 
	SPIUserTransact(hSPI, &trans);
	printf("The config reg now is 0x%02x, default after reset is 0xE3",
		trans.mReadBuffer[1]);
	
CleanUp:
	if (trans.mReadBuffer)
		delete [] trans.mReadBuffer;
	if (trans.mWriteBuffer)
		delete [] trans.mWriteBuffer;
}